#!/bin/exu
--with trace
--------------------------------------------------------------------------------
--
-- Name:         EuDoc - Euphoria documentation program
-- Author:       D. Newhall
-- Version:      0.3
-- License:      Mozilla
-- Created:      Feb-7-2005
-- Updated:      Feb-14-2005
-- Updated:      Jul-13-2005
-- Updated:      Jul-29-2005
-- Updated:      Aug-5-2005   Version 0.1
-- Updated:      Aug-12-2005  Version 0.2
-- Updated:      Sept-18-2005 Version 0.3
--                             Fixed bug where first character would get
--                              removed if right after ':'
-- Updated:      Oct-8-2005   Version 0.4
--                             Added command line args and debugging control
-- Intro:
--
--   To get the full documentation for EuDoc please type "ex EuDoc.e EuDocLib.e"
--   at the command prompt (substituting "exu" or "exw" for "ex" if necessary).
--
--------------------------------------------------------------------------------


-- Possible future features:
--
--   Only global routines and such get documented
--   Type checking for documented things
--   "Input:", "Output:", "Requires:" tags (suggestions and comments welcome)
--   Multiple files into one HTML file
--   GUI version
--   Fix minor formatting bug related to spaces after ':' getting removed when
--     testing for tag
--   Syntax highlighting for Example text
--   Bug fixes  :P



-- Include library file

include EuDocLib.e


-- Print program title

puts(1, "EuDoc - Euphoria Documentation generator")
printf(1, "  ver. %s\n\n", {EUDOC_VERSION})



-- Note: All routines/constants/global variables are defined in EuDocLib.e
-- except...

procedure print_help(integer fn)
    puts(fn, "Usage: ex EuDoc.ex [-h|d|o|n] file [file2 ...]\n\n")
    puts(fn, "  -h\tPrints this help information.\n")
    puts(fn, "  -d\tPrints debugging information to a file.\n")
    puts(fn, "  -o\tOnly ouputs debugging information (not HTML generated).\n")
    puts(fn, "  -n\tShow programmer notes in documentation.\n")
    puts(fn, "  -r\tIgnore routine data (useful for non-library programs).\n")
    puts(fn, "\n")
    puts(fn, "For more documentation run EuDoc on EuDocLib.e.\n\n")
    puts(fn, "  ex EuDoc.ex EuDocLib.e\n")
end procedure



-- Variables

sequence filenums,   -- File handles for files (used to be in EuDocLib)
         cmd,        -- Command line contents
         debug_file  -- Name of file to output debug info to

object line,         -- Line read from file
       output_file   -- Name/file numbers for output file

integer fn,          -- Current file number
	index,       -- Temporary index
	index2,      -- Temporary index,
	cmd_start,   -- Starting index for files passed on command line
	debug        -- Turns debugging on or off



-- Get arguments off the command line 
-- Note: This all used to be from get_command_line() in EuDocLib

cmd = command_line()

if length(cmd) < 3 then
    puts(2, "Error: Not enough arguments specified.\n\n")
    print_help(2)
    abort(1)
elsif length(cmd) > 23 then
    error("Only 23 files can be documented at once.")
end if

cmd_start = 4

if equal(cmd[3], "-h") then
    print_help(1)
    abort(0)
elsif equal(cmd[3], "-d") then
    debug = 1
elsif equal(cmd[3], "-o") then
    debug = 2
elsif equal(cmd[3], "-n") then
    Show_notes = 1
    debug = 0
elsif equal(cmd[3], "-r") then
    Ignore_routines = 1
    debug = 0
else
    debug = 0
    cmd_start = 3
end if


filenums = {}

-- Takes the file names from the command line and opens them
for i = cmd_start to length(cmd) do
    fn = open(cmd[i], "r")
    if fn = -1 then
        printf(2, "Error: file %s doesn't exist.\n", {cmd[i]})
        abort(1)
    else
        filenums &= fn
    end if
end for



-- Go through files passed and output documentation

for i=1 to length(filenums) do

   -- Empty info holding variables for initialization/next file
   empty_data(1)

    -- Set/reset Parsing to FALSE
    Parsing = FALSE

    -- Get next file number
    fn = filenums[i]

    -- Read file line by line
    line = gets(fn)
    while sequence(line) do

	-- Get index of start of comment text
	index = comment(line)

	if index > 0 then -- If line is a comment
	    index2 = match(":", line[index..length(line)])
	    if index2 > 0 then -- If there's a ':' in the line
		-- Next line is confusing since index2 is an offset of index
		read_line(
		  line[index..(index + index2 - 1)],
		  line[(index + index2)..length(line)] )
	    else -- No tag
		if Parsing = TRUE then
		    read_line("", line)
		end if
	    end if
	    
	-- Change Parsing to FALSE if line isn't a comment
	else
	    Parsing = FALSE
	end if

	line = gets(fn)

    end while
    

    -- If debug = 2 (only output debug file) skip this part
    if debug < 2 then

        -- Prompt for HTML file until valid and opened
        output_file = -1

        while atom(output_file) do
        
            -- Check for type of problem with output_file
            if output_file = 0 then     -- Over 11 characters
                puts(2, "File name is too long (8 + 3 chars max)." &
                  " Try again.\n\n")
            elsif output_file = 1 then  -- Extension too long
                puts(2, "File extension is too long (3 chars max)." &
                  " Try again.\n\n")
            elsif output_file = 2 then  -- output_file can't be opened
                printf(2, "File \"%s\" already exists or can not be created" &
                  " for output.\n", {output_file})
            elsif output_file = 2 then  -- list_file can't be opened
                puts(2, "File containing the alphabetical listing of routines" &
                  " can not be created for output.\n")
            end if
    
            output_file = prompt_string("Name of file to output HTML to: ")
            output_file = open_file(output_file)
    
        end while
    
        -- Generate HTML documentation from accumulated data
        generate_html(output_file)

    end if

    -- Call debugging routine if debug = 1 or 2
    if debug >= 1 then
        debug_file = prompt_string("Name of file to output debugging" &
          " information to: ")
        close(fn)  -- Close original file parsed
        fn = open(debug_file, "w")
        if fn = -1 then
            error(sprintf("Debug file \"%s\" can not be opened for output.",
              debug_file))
        end if
        print_data(debug_file)
    end if

    -- Close file after done
    close(fn)

end for
